@@ -140,7 +140,7 @@ QString MyMoneyStatementReader::Private::nameToId(const QString& name, MyMoneyAc
     newAccount.setName(name) ;
     int pos;
     // check for ':' in the name and use it as separator for a hierarchy
-    while ((pos = newAccount.name().indexOf(MyMoneyFile::AccountSeperator)) != -1) {
+    while ((pos = newAccount.name().indexOf(MyMoneyFile::AccountSeparator)) != -1) {
       QString part = newAccount.name().left(pos);
       QString remainder = newAccount.name().mid(pos + 1);
       const MyMoneyAccount& existingAccount = file->subAccountByName(parentAccount, part);
@@ -271,12 +271,12 @@ void MyMoneyStatementReader::Private::assignUniqueBankID(MyMoneySplit& s, const
 void MyMoneyStatementReader::Private::setupPrice(MyMoneySplit &s, const MyMoneyAccount &splitAccount, const MyMoneyAccount &transactionAccount, const QDate &postDate)
 {
   if (transactionAccount.currencyId() != splitAccount.currencyId()) {
-    // a currency converstion is needed asume that split has already a proper value
+    // a currency converstion is needed assume that split has already a proper value
     MyMoneyFile* file = MyMoneyFile::instance();
     MyMoneySecurity toCurrency = file->security(splitAccount.currencyId());
     MyMoneySecurity fromCurrency = file->security(transactionAccount.currencyId());
     // get the price for the transaction's date
-    MyMoneyPrice price = file->price(fromCurrency.id(), toCurrency.id(), postDate);
+    const MyMoneyPrice &price = file->price(fromCurrency.id(), toCurrency.id(), postDate);
     // if the price is valid calculate the shares
     if (price.isValid()) {
       const int fract = splitAccount.fraction(toCurrency);
@@ -305,7 +305,7 @@ MyMoneyStatementReader::~MyMoneyStatementReader()
   delete d;
 }
 
-bool MyMoneyStatementReader::anyTransactionAdded(void) const
+bool MyMoneyStatementReader::anyTransactionAdded() const
 {
   return (d->transactionsAdded != 0) ? true : false;
 }
@@ -368,7 +368,7 @@ bool MyMoneyStatementReader::import(const MyMoneyStatement& s, QStringList& mess
         m_account.setAccountType(MyMoneyAccount::CreditCard);
         break;
       default:
-        m_account.setAccountType(MyMoneyAccount::Checkings);
+        m_account.setAccountType(MyMoneyAccount::UnknownAccountType);
         break;
     }
 
@@ -570,10 +570,10 @@ void MyMoneyStatementReader::processSecurityEntry(const MyMoneyStatement::Securi
   if (security.id().isEmpty()) {
     security.setName(sec_in.m_strName);
     security.setTradingSymbol(sec_in.m_strSymbol);
-    security.setSmallestAccountFraction(1000);
+    security.setSmallestAccountFraction(10000);
     security.setTradingCurrency(file->baseCurrency().id());
     security.setValue("kmm-security-id", sec_in.m_strId);
-    security.setValue("kmm-online-source", "Yahoo");
+    security.setValue("kmm-online-source", "KMyMoney Currency");
     security.setSecurityType(MyMoneySecurity::SECURITY_STOCK);
     MyMoneyFileTransaction ft;
     try {
@@ -655,34 +655,14 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       bool found = false;
       QStringList accounts = thisaccount.accountList();
       QStringList::const_iterator it_account = accounts.constBegin();
+      QString currencyid;
       while (!found && it_account != accounts.constEnd()) {
-        QString currencyid = file->account(*it_account).currencyId();
+        currencyid = file->account(*it_account).currencyId();
         MyMoneySecurity security = file->security(currencyid);
         if (matchNotEmpty(statementTransactionUnderImport.m_strSymbol, security.tradingSymbol()) ||
             matchNotEmpty(statementTransactionUnderImport.m_strSecurity, security.name())) {
           thisaccount = file->account(*it_account);
           found = true;
-
-          // Don't update price if there is no price information contained in the transaction
-          if (statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaCashDividend
-              && statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaShrsin
-              && statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaShrsout) {
-            // update the price, while we're here.  in the future, this should be
-            // an option
-            QString basecurrencyid = file->baseCurrency().id();
-            MyMoneyPrice price = file->price(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted, true);
-            if (!price.isValid()  && ((!statementTransactionUnderImport.m_amount.isZero() && !statementTransactionUnderImport.m_shares.isZero()) || !statementTransactionUnderImport.m_price.isZero())) {
-              MyMoneyPrice newprice;
-              if (!statementTransactionUnderImport.m_price.isZero()) {
-                newprice = MyMoneyPrice(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted,
-                                        statementTransactionUnderImport.m_price.abs(), i18n("Statement Importer"));
-              } else {
-                newprice = MyMoneyPrice(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted,
-                                        (statementTransactionUnderImport.m_amount / statementTransactionUnderImport.m_shares).abs(), i18n("Statement Importer"));
-              }
-              file->addPrice(newprice);
-            }
-          }
         }
 
         ++it_account;
@@ -714,6 +694,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
             thisaccount.setName(security.name());
             thisaccount.setAccountType(MyMoneyAccount::Stock);
             thisaccount.setCurrencyId(security.id());
+            currencyid = thisaccount.currencyId();
 
             file->addAccount(thisaccount, m_account);
             kDebug(0) << Q_FUNC_INFO << ": created account " << thisaccount.id() << " for security " << statementTransactionUnderImport.m_strSecurity << " under account " << m_account.id();
@@ -733,6 +714,26 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
           }
         }
       }
+      // Don't update price if there is no price information contained in the transaction
+      if (statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaCashDividend
+          && statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaShrsin
+          && statementTransactionUnderImport.m_eAction != MyMoneyStatement::Transaction::eaShrsout) {
+        // update the price, while we're here.  in the future, this should be
+        // an option
+        QString basecurrencyid = file->baseCurrency().id();
+        const MyMoneyPrice &price = file->price(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted, true);
+        if (!price.isValid()  && ((!statementTransactionUnderImport.m_amount.isZero() && !statementTransactionUnderImport.m_shares.isZero()) || !statementTransactionUnderImport.m_price.isZero())) {
+          MyMoneyPrice newprice;
+          if (!statementTransactionUnderImport.m_price.isZero()) {
+            newprice = MyMoneyPrice(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted,
+                                    statementTransactionUnderImport.m_price.abs(), i18n("Statement Importer"));
+          } else {
+            newprice = MyMoneyPrice(currencyid, basecurrencyid, statementTransactionUnderImport.m_datePosted,
+                                    (statementTransactionUnderImport.m_amount / statementTransactionUnderImport.m_shares).abs(), i18n("Statement Importer"));
+          }
+          file->addPrice(newprice);
+        }
+      }
     }
     s1.setAccountId(thisaccount.id());
     d->assignUniqueBankID(s1, statementTransactionUnderImport);
@@ -774,8 +775,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       else {//  Ensure category sub-accounts are dealt with properly
         s1.setAccountId(d->interestId(statementTransactionUnderImport.m_strInterestCategory));
       }
-      s1.setShares(-statementTransactionUnderImport.m_amount);
-      s1.setValue(-statementTransactionUnderImport.m_amount);
+      s1.setShares(-statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees);
+      s1.setValue(-statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees);
 
       // Split 2 will be the zero-amount investment split that serves to
       // mark this transaction as a cash dividend and note which stock account
@@ -786,7 +787,11 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       s2.setAccountId(thisaccount.id());
       transactionUnderImport.addSplit(s2);
 
-      transfervalue = statementTransactionUnderImport.m_amount + statementTransactionUnderImport.m_fees;
+      /*  at this point any fees have been taken into account already
+       *  so don't deduct them again.
+       *  BUG 322381
+       */
+      transfervalue = statementTransactionUnderImport.m_amount;
     } else if (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaInterest) {
       if (statementTransactionUnderImport.m_strInterestCategory.isEmpty())
         s1.setAccountId(d->interestId(thisaccount));
@@ -796,8 +801,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
         else
           s1.setAccountId(d->expenseId(statementTransactionUnderImport.m_strInterestCategory));
       }
-      s1.setShares(statementTransactionUnderImport.m_amount);
-      s1.setValue(statementTransactionUnderImport.m_amount);
+      s1.setShares(-statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees);
+      s1.setValue(-statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees);
 
 /// ***********   Add split as per Div       **********
       // Split 2 will be the zero-amount investment split that serves to
@@ -822,29 +827,21 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
 
       transfervalue = statementTransactionUnderImport.m_amount;
 
-    } else if ((statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaBuy) ||
-               (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaSell)) {
-      if (!statementTransactionUnderImport.m_price.isZero()) {
-        s1.setPrice(statementTransactionUnderImport.m_price.abs());
-      } else {
-        if (!statementTransactionUnderImport.m_shares.isZero()) {
-          MyMoneyMoney total = -statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees;
-          s1.setPrice((total / statementTransactionUnderImport.m_shares).abs().convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())));
-        }
-      }
-
+      } else if ((statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaBuy) ||
+                 (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaSell)) {
       s1.setAction(MyMoneySplit::ActionBuyShares);
-
-      // Make sure to setup the sign correctly
-      if (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaBuy) {
-        s1.setShares(statementTransactionUnderImport.m_shares.abs());
-        s1.setValue(s1.value().abs());
-        transfervalue = -(statementTransactionUnderImport.m_amount.abs());
-      } else {
-        s1.setShares(-(statementTransactionUnderImport.m_shares.abs()));
-        s1.setValue(-(s1.value().abs()));
-        transfervalue = statementTransactionUnderImport.m_amount.abs();
+      if (!statementTransactionUnderImport.m_price.isZero())  {
+        s1.setPrice(statementTransactionUnderImport.m_price.abs());
+      } else if (!statementTransactionUnderImport.m_shares.isZero()) {
+        MyMoneyMoney total = statementTransactionUnderImport.m_amount + statementTransactionUnderImport.m_fees.abs();
+        s1.setPrice((total / statementTransactionUnderImport.m_shares).abs().convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())));
       }
+      if (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaBuy)
+          s1.setShares(statementTransactionUnderImport.m_shares.abs());
+      else
+          s1.setShares(-statementTransactionUnderImport.m_shares.abs());
+      s1.setValue(-(statementTransactionUnderImport.m_amount + statementTransactionUnderImport.m_fees.abs()));
+      transfervalue = statementTransactionUnderImport.m_amount;
 
     } else if ((statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaShrsin) ||
                (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaShrsout)) {
@@ -891,7 +888,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
 
   QString payeename = statementTransactionUnderImport.m_strPayee;
   if (!payeename.isEmpty()) {
-    qDebug("Start matching payee");
+    qDebug() << QLatin1String("Start matching payee") << payeename;
     QString payeeid;
     try {
       QList<MyMoneyPayee> pList = file->payeeList();
@@ -976,7 +973,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
         // is called in the context of an automatic procedure it
         // might distract the user.
         payee.setName(payeename);
-        payee.setMatchData(MyMoneyPayee::matchName, true, QStringList());
+        payee.setMatchData(MyMoneyPayee::matchKey, true, QStringList() << QString("^%1$").arg(QRegExp::escape(payeename)));
         if (m_askPayeeCategory) {
           // We use a QPointer because the dialog may get deleted
           // during exec() if the parent of the dialog gets deleted.
@@ -997,8 +994,10 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
           topcontents->addWidget(label1);
 
           AccountNamesFilterProxyModel *filterProxyModel = new AccountNamesFilterProxyModel(this);
+          filterProxyModel->setHideEquityAccounts(!KMyMoneyGlobalSettings::expertMode());
           filterProxyModel->addAccountGroup(MyMoneyAccount::Asset);
           filterProxyModel->addAccountGroup(MyMoneyAccount::Liability);
+          filterProxyModel->addAccountGroup(MyMoneyAccount::Equity);
           filterProxyModel->addAccountGroup(MyMoneyAccount::Income);
           filterProxyModel->addAccountGroup(MyMoneyAccount::Expense);
           filterProxyModel->setSourceModel(Models::instance()->accountsModel());
@@ -1072,6 +1071,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
         s.setShares(-s1.shares());
         s.setValue(-s1.value());
         s.setAccountId(payeeObj.defaultAccountId());
+        s.setMemo(transactionUnderImport.memo());
+        s.setPayeeId(payeeid);
         d->setupPrice(s, splitAccount, m_account, statementTransactionUnderImport.m_datePosted);
         transactionUnderImport.addSplit(s);
         file->addVATSplit(transactionUnderImport, m_account, splitAccount, statementTransactionUnderImport.m_amount);
@@ -1285,7 +1286,19 @@ bool MyMoneyStatementReader::selectOrCreateAccount(const SelectCreateMode /*mode
                 "create a new account by pressing the <b>Create</b> button.");
   }
 
-  KMyMoneyUtils::categoryTypeE type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::asset | KMyMoneyUtils::liability);
+  KMyMoneyUtils::categoryTypeE type;
+  if (account.accountType() == MyMoneyAccount::Checkings) {
+    type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::checking);
+  } else if (account.accountType() == MyMoneyAccount::Savings) {
+    type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::savings);
+  } else if (account.accountType() == MyMoneyAccount::Investment) {
+    type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::investment);
+  } else if (account.accountType() == MyMoneyAccount::CreditCard) {
+    type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::creditCard);
+  } else {
+    type = static_cast<KMyMoneyUtils::categoryTypeE>(KMyMoneyUtils::asset | KMyMoneyUtils::liability);
+  }
+
   QPointer<KAccountSelectDlg> accountSelect = new KAccountSelectDlg(type, "StatementImport", kmymoney);
   accountSelect->setHeader(i18n("Import transactions"));
   accountSelect->setDescription(msg);
