@@ -42,6 +42,7 @@
 #include <KConfigGroup>
 #include <kprocess.h>
 #include <kencodingprober.h>
+
 // ----------------------------------------------------------------------------
 // Project Headers
 
@@ -52,16 +53,40 @@
 QString WebPriceQuote::m_financeQuoteScriptPath;
 QStringList WebPriceQuote::m_financeQuoteSources;
 
-WebPriceQuote::WebPriceQuote(QObject* _parent):
-    QObject(_parent)
+class WebPriceQuote::Private
 {
-  m_financeQuoteScriptPath =
-    KGlobal::dirs()->findResource("appdata", QString("misc/financequote.pl"));
-  connect(&m_filter, SIGNAL(processExited(QString)), this, SLOT(slotParseQuote(QString)));
+public:
+  WebPriceQuoteProcess m_filter;
+  QString m_quoteData;
+  QString m_symbol;
+  QString m_id;
+  QDate m_date;
+  double m_price;
+  WebPriceQuoteSource m_source;
+
+  static int dbgArea() {
+    static int s_area = KDebug::registerArea("KMyMoney (WebPriceQuote)");
+    return s_area;
+  }
+};
+
+WebPriceQuote::WebPriceQuote(QObject* _parent):
+    QObject(_parent),
+    d(new Private)
+{
+  // only do this once (I know, it is not thread safe, but it should
+  // always yield the same result so we don't do any semaphore foo here)
+  if (m_financeQuoteScriptPath.isEmpty()) {
+    m_financeQuoteScriptPath = KGlobal::dirs()->findResource("appdata",
+                               QString("misc/financequote.pl"));
+  }
+  connect(&d->m_filter, SIGNAL(processExited(QString)), this, SLOT(slotParseQuote(QString)));
 }
 
 WebPriceQuote::~WebPriceQuote()
-{ }
+{
+  delete d;
+}
 
 bool WebPriceQuote::launch(const QString& _symbol, const QString& _id, const QString& _sourcename)
 {
@@ -74,8 +99,8 @@ bool WebPriceQuote::launch(const QString& _symbol, const QString& _id, const QSt
 bool WebPriceQuote::launchNative(const QString& _symbol, const QString& _id, const QString& _sourcename)
 {
   bool result = true;
-  m_symbol = _symbol;
-  m_id = _id;
+  d->m_symbol = _symbol;
+  d->m_id = _id;
 
 //   emit status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol,_id));
 
@@ -85,37 +110,37 @@ bool WebPriceQuote::launchNative(const QString& _symbol, const QString& _id, con
     sourcename = "Yahoo";
 
   if (quoteSources().contains(sourcename))
-    m_source = WebPriceQuoteSource(sourcename);
+    d->m_source = WebPriceQuoteSource(sourcename);
   else
     emit error(i18n("Source <placeholder>%1</placeholder> does not exist.", sourcename));
 
   KUrl url;
 
   // if the source has room for TWO symbols..
-  if (m_source.m_url.contains("%2")) {
+  if (d->m_source.m_url.contains("%2")) {
     // this is a two-symbol quote.  split the symbol into two.  valid symbol
     // characters are: 0-9, A-Z and the dot.  anything else is a separator
     QRegExp splitrx("([0-9a-z\\.]+)[^a-z0-9]+([0-9a-z\\.]+)", Qt::CaseInsensitive);
     // if we've truly found 2 symbols delimited this way...
-    if (splitrx.indexIn(m_symbol) != -1)
-      url = KUrl(m_source.m_url.arg(splitrx.cap(1), splitrx.cap(2)));
+    if (splitrx.indexIn(d->m_symbol) != -1)
+      url = KUrl(d->m_source.m_url.arg(splitrx.cap(1), splitrx.cap(2)));
     else
-      kDebug(2) << "WebPriceQuote::launch() did not find 2 symbols";
+      kDebug(Private::dbgArea()) << "WebPriceQuote::launch() did not find 2 symbols";
   } else
     // a regular one-symbol quote
-    url = KUrl(m_source.m_url.arg(m_symbol));
+    url = KUrl(d->m_source.m_url.arg(d->m_symbol));
 
   if (url.isLocalFile()) {
     emit status(i18nc("The process x is executing", "Executing %1...", url.toLocalFile()));
 
-    m_filter.clearProgram();
-    m_filter << url.toLocalFile().split(' ', QString::SkipEmptyParts);
-    m_filter.setSymbol(m_symbol);
+    d->m_filter.clearProgram();
+    d->m_filter << url.toLocalFile().split(' ', QString::SkipEmptyParts);
+    d->m_filter.setSymbol(d->m_symbol);
 
-    m_filter.setOutputChannelMode(KProcess::MergedChannels);
-    m_filter.start();
+    d->m_filter.setOutputChannelMode(KProcess::MergedChannels);
+    d->m_filter.start();
 
-    if (m_filter.waitForStarted()) {
+    if (d->m_filter.waitForStarted()) {
       result = true;
     } else {
       emit error(i18n("Unable to launch: %1", url.toLocalFile()));
@@ -126,8 +151,8 @@ bool WebPriceQuote::launchNative(const QString& _symbol, const QString& _id, con
 
     QString tmpFile;
     if (KIO::NetAccess::download(url, tmpFile, 0)) {
-      kDebug(2) << "Downloaded " << tmpFile;
-      //kDebug(2) << "Downloaded " << tmpFile << " from " << url;
+      // kDebug(Private::dbgArea()) << "Downloaded " << tmpFile;
+      kDebug(Private::dbgArea()) << "Downloaded" << tmpFile << "from" << url;
       QFile f(tmpFile);
       if (f.open(QIODevice::ReadOnly)) {
         result = true;
@@ -135,7 +160,10 @@ bool WebPriceQuote::launchNative(const QString& _symbol, const QString& _id, con
         QByteArray page = f.readAll();
         KEncodingProber prober(KEncodingProber::Universal);
         prober.feed(page);
-        QString quote = QTextCodec::codecForName(prober.encoding())->toUnicode(page);
+        QTextCodec* codec = QTextCodec::codecForName(prober.encoding());
+        if (!codec)
+          codec = QTextCodec::codecForLocale();
+        QString quote = codec->toUnicode(page);
         f.close();
         slotParseQuote(quote);
       } else {
@@ -155,27 +183,27 @@ bool WebPriceQuote::launchFinanceQuote(const QString& _symbol, const QString& _i
                                        const QString& _sourcename)
 {
   bool result = true;
-  m_symbol = _symbol;
-  m_id = _id;
+  d->m_symbol = _symbol;
+  d->m_id = _id;
   QString FQSource = _sourcename.section(' ', 1);
-  m_source = WebPriceQuoteSource(_sourcename, m_financeQuoteScriptPath,
-                                 "\"([^,\"]*)\",.*",  // symbol regexp
-                                 "[^,]*,[^,]*,\"([^\"]*)\"", // price regexp
-                                 "[^,]*,([^,]*),.*", // date regexp
-                                 "%y-%m-%d"); // date format
+  d->m_source = WebPriceQuoteSource(_sourcename, m_financeQuoteScriptPath,
+                                    "\"([^,\"]*)\",.*",  // symbol regexp
+                                    "[^,]*,[^,]*,\"([^\"]*)\"", // price regexp
+                                    "[^,]*,([^,]*),.*", // date regexp
+                                    "%y-%m-%d"); // date format
 
   //emit status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol,_id));
 
-  m_filter.clearProgram();
-  m_filter << "perl" << m_financeQuoteScriptPath << FQSource << KShell::quoteArg(_symbol);
-  m_filter.setSymbol(m_symbol);
+  d->m_filter.clearProgram();
+  d->m_filter << "perl" << m_financeQuoteScriptPath << FQSource << KShell::quoteArg(_symbol);
+  d->m_filter.setSymbol(d->m_symbol);
   emit status(i18nc("Executing 'script' 'online source' 'investment symbol' ", "Executing %1 %2 %3...", m_financeQuoteScriptPath, FQSource, _symbol));
 
-  m_filter.setOutputChannelMode(KProcess::MergedChannels);
-  m_filter.start();
+  d->m_filter.setOutputChannelMode(KProcess::MergedChannels);
+  d->m_filter.start();
 
   // This seems to work best if we just block until done.
-  if (m_filter.waitForFinished()) {
+  if (d->m_filter.waitForFinished()) {
     result = true;
   } else {
     emit error(i18n("Unable to launch: %1", m_financeQuoteScriptPath));
@@ -188,13 +216,14 @@ bool WebPriceQuote::launchFinanceQuote(const QString& _symbol, const QString& _i
 void WebPriceQuote::slotParseQuote(const QString& _quotedata)
 {
   QString quotedata = _quotedata;
+  d->m_quoteData = quotedata;
   bool gotprice = false;
   bool gotdate = false;
 
-  // kDebug(2) << "WebPriceQuote::slotParseQuote( " << _quotedata << " ) ";
+  kDebug(Private::dbgArea()) << "quotedata" << _quotedata;
 
   if (! quotedata.isEmpty()) {
-    if (!m_source.m_skipStripping) {
+    if (!d->m_source.m_skipStripping) {
       //
       // First, remove extranous non-data elements
       //
@@ -207,23 +236,17 @@ void WebPriceQuote::slotParseQuote(const QString& _quotedata)
 
       // Extra white space
       quotedata = quotedata.simplified();
+      kDebug(Private::dbgArea()) << "stripped text" << quotedata;
     }
 
-#ifdef KMM_DEBUG
-    // Enable to get a look at the data coming back from the source after it's stripped
-    QFile file("stripped.txt");
-    if (file.open(QIODevice::WriteOnly)) {
-      QTextStream(&file) << quotedata;
-      file.close();
+    QRegExp symbolRegExp(d->m_source.m_sym);
+    QRegExp dateRegExp(d->m_source.m_date);
+    QRegExp priceRegExp(d->m_source.m_price);
+
+    if (symbolRegExp.indexIn(quotedata) > -1) {
+      kDebug(Private::dbgArea()) << "Symbol" << symbolRegExp.cap(1);
+      emit status(i18n("Symbol found: '%1'", symbolRegExp.cap(1)));
     }
-#endif
-
-    QRegExp symbolRegExp(m_source.m_sym);
-    QRegExp dateRegExp(m_source.m_date);
-    QRegExp priceRegExp(m_source.m_price);
-
-    if (symbolRegExp.indexIn(quotedata) > -1)
-      emit status(i18n("Symbol found: %1", symbolRegExp.cap(1)));
 
     if (priceRegExp.indexIn(quotedata) > -1) {
       gotprice = true;
@@ -247,79 +270,55 @@ void WebPriceQuote::slotParseQuote(const QString& _quotedata)
         pos = pricestr.lastIndexOf(QRegExp("\\D"), pos);
       }
 
-      m_price = pricestr.toDouble();
-      emit status(i18n("Price found: %1 (%2)", pricestr, m_price));
+      d->m_price = pricestr.toDouble();
+      kDebug(Private::dbgArea()) << "Price" << pricestr;
+      emit status(i18n("Price found: '%1' (%2)", pricestr, d->m_price));
     }
 
     if (dateRegExp.indexIn(quotedata) > -1) {
       QString datestr = dateRegExp.cap(1);
 
-      MyMoneyDateFormat dateparse(m_source.m_dateformat);
+      MyMoneyDateFormat dateparse(d->m_source.m_dateformat);
       try {
-        m_date = dateparse.convertString(datestr, false /*strict*/);
+        d->m_date = dateparse.convertString(datestr, false /*strict*/);
         gotdate = true;
-        emit status(i18n("Date found: %1", m_date.toString()));;
-      } catch (MyMoneyException* e) {
-        // emit error(i18n("Unable to parse date %1 using format %2: %3").arg(datestr,dateparse.format(),e->what()));
-        m_date = QDate::currentDate();
+        kDebug(Private::dbgArea()) << "Date" << datestr;
+        emit status(i18n("Date found: '%1'", d->m_date.toString()));;
+      } catch (const MyMoneyException &) {
+        // emit error(i18n("Unable to parse date %1 using format %2: %3").arg(datestr,dateparse.format(),e.what()));
+        d->m_date = QDate::currentDate();
         gotdate = true;
-        delete e;
       }
     }
 
     if (gotprice && gotdate) {
-      emit quote(m_id, m_symbol, m_date, m_price);
+      emit quote(d->m_id, d->m_symbol, d->m_date, d->m_price);
     } else {
-      emit error(i18n("Unable to update price for %1 (no price or no date)", m_symbol));
-      emit failed(m_id, m_symbol);
+      emit error(i18n("Unable to update price for %1 (no price or no date)", d->m_symbol));
+      emit failed(d->m_id, d->m_symbol);
     }
   } else {
-    emit error(i18n("Unable to update price for %1 (empty quote data)", m_symbol));
-    emit failed(m_id, m_symbol);
+    emit error(i18n("Unable to update price for %1 (empty quote data)", d->m_symbol));
+    emit failed(d->m_id, d->m_symbol);
   }
 }
 
-const QMap<QString, WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void)
+const QMap<QString, WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources()
 {
   QMap<QString, WebPriceQuoteSource> result;
 
-  result["Yahoo"] = WebPriceQuoteSource("Yahoo",
-                                        "http://finance.yahoo.com/d/quotes.csv?s=%1&f=sl1d1",
-                                        "\"([^,\"]*)\",.*",  // symbolregexp
-                                        "[^,]*,([^,]*),.*", // priceregexp
-                                        "[^,]*,[^,]*,\"([^\"]*)\"", // dateregexp
-                                        "%m %d %y" // dateformat
-                                       );
-
-  result["Yahoo Currency"] = WebPriceQuoteSource("Yahoo Currency",
-                             "http://finance.yahoo.com/d/quotes.csv?s=%1%2=X&f=sl1d1",
-                             "\"([^,\"]*)\",.*",  // symbolregexp
-                             "[^,]*,([^,]*),.*", // priceregexp
-                             "[^,]*,[^,]*,\"([^\"]*)\"", // dateregexp
-                             "%m %d %y" // dateformat
-                                                );
-
-  // 2009-08-20 Yahoo UK has no quotes and has comma separators
-  // sl1d1 format for Yahoo UK doesn't seem to give a date ever
-  // sl1d3 gives US locale time (9:99pm) and date (mm/dd/yyyy)
-  result["Yahoo UK"] = WebPriceQuoteSource("Yahoo UK",
-                       "http://uk.finance.yahoo.com/d/quotes.csv?s=%1&f=sl1d3",
-                       "^([^,]*),.*",  // symbolregexp
-                       "^[^,]*,([^,]*),.*", // priceregexp
-                       "^[^,]*,[^,]*, [^ ]* (../../....).*", // dateregexp
-                       "%m/%d/%y" // dateformat
+  // Use fx-rate.net as the standard currency exchange rate source until
+  // we have the capability to use more than one source. Use a neutral
+  // name for the source.
+  result["KMyMoney Currency"] = WebPriceQuoteSource("KMyMoney Currency",
+                                          "https://fx-rate.net/%1/%2",
+                                           QString(),  // symbolregexp
+                                          "1[ a-zA-Z]+=</span><br */?> *(\\d+\\.\\d+)",
+                                          "updated\\s\\d+:\\d+:\\d+\\(\\w+\\)\\s+(\\d{1,2}/\\d{2}/\\d{4})",
+                                          "%d/%m/%y",
+                                          true // skip HTML stripping
                                           );
 
-  // sl1d1 format for Yahoo France doesn't seem to give a date ever
-  // sl1d3 gives us time (99h99) and date
-  result["Yahoo France"] = WebPriceQuoteSource("Yahoo France",
-                           "http://fr.finance.yahoo.com/d/quotes.csv?s=%1&f=sl1d3",
-                           "([^;]*).*",             // symbolregexp
-                           "[^;]*.([^;]*),*",       // priceregexp
-                           "[^;]*.[^;]*...h...([^;]*)", // dateregexp
-                           "%d/%m/%y"               // dateformat
-                                              );
-
   result["Globe & Mail"] = WebPriceQuoteSource("Globe & Mail",
                            "http://globefunddb.theglobeandmail.com/gishome/plsql/gis.price_history?pi_fund_id=%1",
                            QString(),  // symbolregexp
@@ -362,28 +361,19 @@ const QMap<QString, WebPriceQuoteSource> WebPriceQuote::defaultQuoteSources(void
       ", (\\d+\\D+\\d+\\D+\\d+)", // dateregexp
       "%d %m %y" // dateformat
                                                                 );
-
-  // This quote source provided by Peter Lord
-  // The trading symbol will normally be the SEDOL (see wikipedia) but
-  // the flexibility presently (1/2008) in the code will allow use of
-  // the ISIN or MEXID (FT specific) codes
+  // This quote source provided by e-mail and should replace the previous one:
+  // From: David Houlden <djhoulden@gmail.com>
+  // To: kmymoney@kde.org
+  // Date: Sat, 6 Apr 2013 13:22:45 +0100
   result["Financial Times UK Funds"] = WebPriceQuoteSource("Financial Times UK Funds",
-                                       "http://funds.ft.com/funds/simpleSearch.do?searchArea=%&search=%1",
-                                       "SEDOL[\\ ]*(\\d+.\\d+)", // symbol regexp
-                                       "\\(GBX\\)[\\ ]*([0-9,]*.\\d+)[\\ ]*", // price regexp
-                                       "Valuation date:[\\ ]*(\\d+/\\d+/\\d+)", // date regexp
-                                       "%d/%m/%y" // date format
+                                       "http://funds.ft.com/uk/Tearsheet/Summary?s=%1",
+                                       "data-display-symbol=\"(.*):", // symbol regexp
+                                       "class=\"text first\">([\\d,]*\\d+\\.\\d+)</td>", // price regexp
+                                       "As of market close\\ (.*)\\.", // date regexp
+                                       "%m %d %y", // date format
+                                       true // skip HTML stripping
                                                           );
 
-  // This quote source provided by Danny Scott
-  result["Yahoo Canada"] = WebPriceQuoteSource("Yahoo Canada",
-                           "http://ca.finance.yahoo.com/q?s=%1",
-                           "%1", // symbol regexp
-                           "Last Trade: (\\d+\\.\\d+)", // price regexp
-                           "day, (.\\D+\\d+\\D+\\d+)", // date regexp
-                           "%m %d %y" // date format
-                                              );
-
   // (tf2k) The "mpid" is I think the market place id. In this case five
   // stands for Hamburg.
   //
@@ -556,8 +546,6 @@ const QStringList WebPriceQuote::quoteSourcesFinanceQuote()
                                  QString("misc/financequote.pl"));
     }
     FinanceQuoteProcess getList;
-    m_financeQuoteScriptPath =
-      KGlobal::dirs()->findResource("appdata", QString("misc/financequote.pl"));
     getList.launch(m_financeQuoteScriptPath);
     while (!getList.isFinished()) {
       kapp->processEvents();
@@ -594,7 +582,7 @@ WebPriceQuoteSource::WebPriceQuoteSource(const QString& name)
   m_skipStripping = grp.readEntry("SkipStripping", false);
 }
 
-void WebPriceQuoteSource::write(void) const
+void WebPriceQuoteSource::write() const
 {
   KSharedConfigPtr kconfig = KGlobal::config();
   KConfigGroup grp = kconfig->group(QString("Online-Quote-Source-%1").arg(m_name));
@@ -616,7 +604,7 @@ void WebPriceQuoteSource::rename(const QString& name)
   write();
 }
 
-void WebPriceQuoteSource::remove(void) const
+void WebPriceQuoteSource::remove() const
 {
   KSharedConfigPtr kconfig = KGlobal::config();
   kconfig->deleteGroup(QString("Online-Quote-Source-%1").arg(m_name));
@@ -626,7 +614,7 @@ void WebPriceQuoteSource::remove(void) const
 // Helper class to babysit the KProcess used for running the local script in that case
 //
 
-WebPriceQuoteProcess::WebPriceQuoteProcess(void)
+WebPriceQuoteProcess::WebPriceQuoteProcess()
 {
   connect(this, SIGNAL(readyReadStandardOutput()), this, SLOT(slotReceivedDataFromFilter()));
   connect(this, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(slotProcessExited(int,QProcess::ExitStatus)));
@@ -649,7 +637,7 @@ void WebPriceQuoteProcess::slotProcessExited(int /*exitCode*/, QProcess::ExitSta
 // Helper class to babysit the KProcess used for running the Finance Quote sources script
 //
 
-FinanceQuoteProcess::FinanceQuoteProcess(void)
+FinanceQuoteProcess::FinanceQuoteProcess()
 {
   m_isDone = false;
   m_string = "";
@@ -786,7 +774,7 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
 
   QRegExp formatrex("%([mdy]+)(\\W+)%([mdy]+)(\\W+)%([mdy]+)", Qt::CaseInsensitive);
   if (formatrex.indexIn(m_format) == -1) {
-    throw new MYMONEYEXCEPTION("Invalid format string");
+    throw MYMONEYEXCEPTION("Invalid format string");
   }
 
   QStringList formatParts;
@@ -814,7 +802,7 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
     inputrex.setPattern("(\\w+)\\W+(\\w+)\\W+(\\w+)");
 
   if (inputrex.indexIn(_in) == -1) {
-    throw new MYMONEYEXCEPTION("Invalid input string");
+    throw MYMONEYEXCEPTION("Invalid input string");
   }
 
   QStringList scannedParts;
@@ -839,7 +827,7 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
         if (digitrex.indexIn(*it_scanned) != -1)
           day = digitrex.cap(1).toUInt(&ok);
         if (!ok || day > 31)
-          throw new MYMONEYEXCEPTION(QString("Invalid day entry: %1").arg(*it_scanned));
+          throw MYMONEYEXCEPTION(QString("Invalid day entry: %1").arg(*it_scanned));
         break;
       case 'm':
         month = (*it_scanned).toUInt(&ok);
@@ -855,18 +843,18 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
         }
 
         if (month < 1 || month > 12)
-          throw new MYMONEYEXCEPTION(QString("Invalid month entry: %1").arg(*it_scanned));
+          throw MYMONEYEXCEPTION(QString("Invalid month entry: %1").arg(*it_scanned));
 
         break;
       case 'y':
         if (_strict && (*it_scanned).length() != (*it_format).length())
-          throw new MYMONEYEXCEPTION(QString("Length of year (%1) does not match expected length (%2).")
-                                     .arg(*it_scanned, *it_format));
+          throw MYMONEYEXCEPTION(QString("Length of year (%1) does not match expected length (%2).")
+                                 .arg(*it_scanned, *it_format));
 
         year = (*it_scanned).toUInt(&ok);
 
         if (!ok)
-          throw new MYMONEYEXCEPTION(QString("Invalid year entry: %1").arg(*it_scanned));
+          throw MYMONEYEXCEPTION(QString("Invalid year entry: %1").arg(*it_scanned));
 
         //
         // 2-digit year case
@@ -884,11 +872,11 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
         }
 
         if (year < 1900)
-          throw new MYMONEYEXCEPTION(QString("Invalid year (%1)").arg(year));
+          throw MYMONEYEXCEPTION(QString("Invalid year (%1)").arg(year));
 
         break;
       default:
-        throw new MYMONEYEXCEPTION("Invalid format character");
+        throw MYMONEYEXCEPTION("Invalid format character");
     }
 
     ++it_scanned;
@@ -896,7 +884,7 @@ const QDate MyMoneyDateFormat::convertString(const QString& _in, bool _strict, u
   }
   QDate result(year, month, day);
   if (! result.isValid())
-    throw new MYMONEYEXCEPTION(QString("Invalid date (yr%1 mo%2 dy%3)").arg(year).arg(month).arg(day));
+    throw MYMONEYEXCEPTION(QString("Invalid date (yr%1 mo%2 dy%3)").arg(year).arg(month).arg(day));
 
   return result;
 }
@@ -917,7 +905,8 @@ convertertest::QuoteReceiver::QuoteReceiver(WebPriceQuote* q, QObject* parent) :
 }
 
 convertertest::QuoteReceiver::~QuoteReceiver()
-{ }
+{
+}
 
 void convertertest::QuoteReceiver::slotGetQuote(const QString&, const QString&, const QDate& d, const double& m)
 {
@@ -926,12 +915,14 @@ void convertertest::QuoteReceiver::slotGetQuote(const QString&, const QString&,
   m_price = MyMoneyMoney(m);
   m_date = d;
 }
+
 void convertertest::QuoteReceiver::slotStatus(const QString& msg)
 {
 //   kDebug(2) << "test::QuoteReceiver::slotStatus( " << msg << " )";
 
   m_statuses += msg;
 }
+
 void convertertest::QuoteReceiver::slotError(const QString& msg)
 {
 //   kDebug(2) << "test::QuoteReceiver::slotError( " << msg << " )";
@@ -939,6 +930,13 @@ void convertertest::QuoteReceiver::slotError(const QString& msg)
   m_errors += msg;
 }
 
-// vim:cin:si:ai:et:ts=2:sw=2:
-
-#include "webpricequote.moc"
+// leave this moc until we will have resolved our dependency issues
+// now 'converter' depends on 'kmymoney' a pointer to the application
+// defined in main.cpp, which makes this static library unusable without
+// the --as-needed linker flag;
+// otherwise the 'moc' methods of this object will be linked into the automoc
+// object file which contains methods from all the other objects from this
+// library, thus even if the --as-needed option is used all objects will be
+// pulled in while linking 'convertertest' which only needs the WebPriceQuote
+// object - spent a whole day investigating this
+#include "moc_webpricequote.cpp"
